// 03KeyLock.cpp : Defines the entry point for the application.
//

#include "stdafx.h"

#include <winuserm.h>
#include <ras.h>

#include "03KeyLockComm.h"
#include "TouchPanel.h"
#include "Util.h"
#include "KeyHookAPI.h"
#include "HookKeyHook.h"
#include "PhoneStatusUtil.h"
#include "KeyLockPref.h"
#include "PowerWatch.h"

/* Defines */
#define WZERO3_KEYLOCK_APP_DBG_CAPTION			_T("03KeyLock")
#define	WZERO3_KEYLOCK_THRESHOLD				(2*1000)
#define	DENWA_WINDOW_TITLE						_T("db ")

#define TIMERID_WAIT_CHECKKEYLOCK		1
#define TIMERID_WAIT_RESUSPEND			2

#define WAIT_TIME_CHECKKEYLOCK		200
#define WAIT_TIME_RESUSPEND			(6*1000)


// HookKeyHook features
#define	HOOKKEYHOOK_SETTING_REGKEY	_T("Software\\HookKeyHook")
#define	HOOKKEYHOOK_OFFHOOK_PATH	_T("OffHookKey_cmd")
#define	HOOKKEYHOOK_OFFHOOK_ARG		_T("OffHookKey_arg")
#define	HOOKKEYHOOK_ONHOOK_PATH		_T("OnHookKey_cmd")
#define	HOOKKEYHOOK_ONHOOK_ARG		_T("OnHookKey_arg")
#define	HOOKKEYHOOK_SLIDEOPEN_PATH	_T("SlideOpen_cmd")
#define	HOOKKEYHOOK_SLIDEOPEN_ARG	_T("SlideOpen_arg")
#define	HOOKKEYHOOK_SLIDECLOSE_PATH	_T("SlideClose_cmd")
#define	HOOKKEYHOOK_SLIDECLOSE_ARG	_T("SlideClose_arg")


/* Typedefs */
typedef enum {
	KEYLOCK_STATE_OFF = 0,
	KEYLOCK_STATE_ON,
} KEYLOCK_STATE;

/* ProtoTypes */
static LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);


/* Global Variables */
static	HINSTANCE		gs_hInst		= NULL;
static	HHOOK			gs_hHook		= NULL;
static	KEYLOCK_STATE	gs_LockState	= KEYLOCK_STATE_OFF;
static	CKeyLockPref	gs_KeyLockPref;

/////////////////////////////////////////

static
BOOL
IsPPPConnected()
{
	BOOL		bRet;
	RASCONN		rasConn;
	DWORD		dwSize = 0;
	DWORD		dwCnt = 0;

	bRet = FALSE;

	memset(&rasConn, 0, sizeof(rasConn));
	rasConn.dwSize = sizeof(RASCONN);
	dwSize = sizeof(RASCONN);

	if (!RasEnumConnections(&rasConn, &dwSize, &dwCnt)) {
		if (dwCnt > 0) {
			RASCONNSTATUS	rasConnStatus;

			memset(&rasConnStatus, 0, sizeof(rasConnStatus));
			rasConnStatus.dwSize=sizeof(RASCONNSTATUS);
			if(!RasGetConnectStatus(rasConn.hrasconn, &rasConnStatus)) {
				if(!wcscmp(rasConnStatus.szDeviceType, RASDT_Modem)) {
					bRet = TRUE;
				}
			}
		}
	}

	return bRet;
}

static
BOOL
IsOutLookReceived()
{
	BOOL	bRet;
	DWORD	dwVal;
	HKEY	hKey = NULL;

	bRet = FALSE;

	hKey = OpenPhoneStatus();
	if (hKey != NULL) {

		dwVal = 0;

		if (GetPhoneStatus(hKey, 15, &dwVal)) {
			if (dwVal == 1) {
				bRet = TRUE;
			}
		}

		ClosePhoneStatus(hKey);
		hKey = NULL;
	}

	return bRet;
}

static
BOOL
IsDenwaActive()
{
	BOOL	bRet;
	HWND	hWnd;
	TCHAR	szCaption[32];

	bRet = FALSE;

	hWnd = GetForegroundWindow();
	if (hWnd != NULL) {
		memset(szCaption, 0, sizeof(szCaption));
		GetWindowText(hWnd, szCaption, sizeof(szCaption)/sizeof(szCaption[0]));
		if (_tcscmp(szCaption, DENWA_WINDOW_TITLE) == 0) {
			bRet = TRUE;
		}
	}

	return bRet;
}


static
void
doSuspend()
{
	keybd_event(VK_OFF, 0, KEYEVENTF_SILENT, 0);
	keybd_event(VK_OFF, 0, KEYEVENTF_KEYUP|KEYEVENTF_SILENT, 0);
}

static
void
WINAPI
NotifyResumeProc(
	void* pParam
)
{
	HWND	hWnd = NULL;

	hWnd = (HWND)pParam;
	if (hWnd != NULL) {
		PostMessage(hWnd, WM_USER_NOTIFY_RESUME, 0, 0);
	}
}

static
BOOL
InvokeApp(
	LPTSTR	pPath,
	LPTSTR	pArg
)
{
	BOOL				bRet;
	PROCESS_INFORMATION pi;

	bRet = FALSE;	// default return value

	if (pPath == NULL) {
		return FALSE;
	}

	memset(&pi, 0, sizeof(pi));
	if (pArg != NULL) {
		bRet = CreateProcess(pPath, pArg, NULL, NULL, FALSE, 0, NULL, NULL, NULL, &pi);
	}
	else {
		bRet = CreateProcess(pPath, NULL, NULL, NULL, FALSE, 0, NULL, NULL, NULL, &pi);
	}

	if (bRet) {
		CloseHandle(pi.hThread);
		CloseHandle(pi.hProcess);
	}

	return bRet;
}

static
BOOL
ExecUserAction(
	LPTSTR	szRegKeyRoot,
	LPTSTR	szCmdValueName,
	LPTSTR	szArgValueName
)
{
	BOOL	bRet;
	LONG	lResult;
	HKEY	hKey = NULL;

	bRet = FALSE;	// default return value

	lResult = RegOpenKeyEx(HKEY_CURRENT_USER, szRegKeyRoot, 0, KEY_ALL_ACCESS, &hKey);

	if (lResult == ERROR_SUCCESS) {
		TCHAR	szPath[MAX_PATH+1];
		TCHAR	szArg[MAX_PATH+1];
		DWORD	dwType = 0;
		DWORD	dwSize = 0;

		memset(szPath, 0, sizeof(szPath));
		dwSize = sizeof(szPath);
		lResult = RegQueryValueEx(hKey, szCmdValueName, NULL, &dwType, (LPBYTE)szPath, &dwSize);
		if (lResult == ERROR_SUCCESS && _tcslen(szPath) != 0) {
			BOOL				bCreateSucc = FALSE;
			PROCESS_INFORMATION pi;

			memset(&pi, 0, sizeof(pi));

			memset(szArg, 0, sizeof(szArg));
			dwSize = sizeof(szArg);
			lResult = RegQueryValueEx(hKey, szArgValueName, NULL, &dwType, (LPBYTE)szArg, &dwSize);
			if (lResult == ERROR_SUCCESS && _tcslen(szArg) != 0) {
				bRet = InvokeApp(szPath, szArg);
			}
			else {
				bRet = InvokeApp(szPath, NULL);
			}
		}

		RegCloseKey(hKey);
		hKey = NULL;
	}

	return bRet;
}


static
BOOL
SetLockIcon()
{
	BOOL	bRet;
	HKEY	hKey = NULL;

	bRet = FALSE;

	hKey = OpenPhoneStatus();
	if (hKey != NULL) {

		bRet = SetPhoneStatus(hKey, 0, 1);	// Status0̒l1

		ClosePhoneStatus(hKey);
		hKey = NULL;
	}

	return bRet;
}

static
UnsetLockIcon()
{
	BOOL	bRet;
	HKEY	hKey = NULL;

	bRet = FALSE;

	hKey = OpenPhoneStatus();
	if (hKey != NULL) {

		bRet = SetPhoneStatus(hKey, 0, 0);	// Status0̒l0

		ClosePhoneStatus(hKey);
		hKey = NULL;
	}

	return bRet;
}

static
void
doKeyLock()
{
	if (TP_IsTouchPanelEnable()) {
		TP_EnableTouchPanel(FALSE);
	}
	gs_LockState = KEYLOCK_STATE_ON;

	SetLockIcon();
}

static
void
doKeyUnlock()
{
	if (!TP_IsTouchPanelEnable()) {
		TP_EnableTouchPanel(TRUE);
	}
	gs_LockState = KEYLOCK_STATE_OFF;

	UnsetLockIcon();
}

static
KEYLOCK_STATE
GetKeyLockState(
)
{
	// rȂĂv...
	return gs_LockState;
}

static
BOOL
IsKeyLock(
)
{
	return (gs_LockState == KEYLOCK_STATE_ON);
}

static
BOOL
StartHook(
  HOOKPROC	pfnHookProc	// IN
)
{
	BOOL	bRet;
	HHOOK	hHook = NULL;

	bRet = FALSE;

	if (gs_hHook == NULL) {
		// hookς
		hHook = SetWindowsHookExW(WH_KEYBOARD_LL, pfnHookProc, NULL, 0);
		if (hHook != NULL) {
			bRet = TRUE;
			gs_hHook = hHook;
		}
	}
	return bRet;
}

static
BOOL
StopHook()
{
	BOOL	bRet;

	bRet = FALSE;
	if (gs_hHook != NULL) {
		UnhookWindowsHookEx(gs_hHook);
		gs_hHook = NULL;

		bRet = TRUE;
	}

	return bRet;
}


static
void
NotifyUnlock()
{
	static	HWND	s_hWnd			= NULL;

	if (s_hWnd == NULL) {
		s_hWnd = FindWindow(WZERO3_KEYLOCK_WINDOW_CLASS_NAME, WZERO3_KEYLOCK_WINDOW_TITLE_NAME);
	}

	if (s_hWnd != NULL) {
		// CEBhEփbNʒm
		PostMessage(s_hWnd, WM_USER_KEYLOCK_CONTROL, KEYLOCK_CONTROL_OFF, 0);
	}
	else {
		// CEBhEȂꍇ
		// 傤Ȃ̂ŁC̃ReLXg烍bN
		doKeyUnlock();
	}
}


static
void
NotifyExec(
	DWORD dwAction
)
{
	static	HWND	s_hWnd			= NULL;

	if (s_hWnd == NULL) {
		s_hWnd = FindWindow(WZERO3_KEYLOCK_WINDOW_CLASS_NAME, WZERO3_KEYLOCK_WINDOW_TITLE_NAME);
	}

	if (s_hWnd != NULL) {
		// CEBhEփAvsʒm
		PostMessage(s_hWnd, WM_USER_EXEC_IN_KEYLOCK, dwAction, 0);
	}
}

static
void
NotifyPreventSuspend(
)
{
	static	HWND	s_hWnd			= NULL;

	if (s_hWnd == NULL) {
		s_hWnd = FindWindow(WZERO3_KEYLOCK_WINDOW_CLASS_NAME, WZERO3_KEYLOCK_WINDOW_TITLE_NAME);
	}

	if (s_hWnd != NULL) {
		// CEBhEփAvsʒm
		PostMessage(s_hWnd, WM_USER_PREVENT_SUSPEND, 0, 0);
	}
}


#define	STATE_IN_INIT				0
#define	STATE_IN_FIRSTBTN_PRESSED	1
#define	STATE_IN_IGNORE_REST		2

static
LRESULT
CALLBACK
HookProc(
	int		nCode,
	WPARAM	wParam,
	LPARAM	lParam
)
{
	int		i;
	static	DWORD	s_dwStartTick	= 0;
	static	DWORD	s_dwState		= STATE_IN_INIT;
	static	CKeyLockPref s_KeyLockPref;
	const static struct {
		DWORD	vkCode;
		DWORD	dwExecFlag;
	} execTable[] = {
		{	VK_F1,		WZERO3_KEYLOCK_EXEC_VK_F1	},
		{	VK_F2,		WZERO3_KEYLOCK_EXEC_VK_F2	},
		{	VK_APP1,	WZERO3_KEYLOCK_EXEC_VK_APP1	},
		{	VK_APP2,	WZERO3_KEYLOCK_EXEC_VK_APP2	},
		{	VK_APP3,	WZERO3_KEYLOCK_EXEC_VK_APP3	},
	};

	if (nCode != HC_ACTION) {
		return CallNextHookEx(gs_hHook, nCode, wParam, lParam);
	}

	// bNłTXyhڍs\Ƃ
	if (PKBDLLHOOKSTRUCT(lParam)->vkCode == VK_OFF) {
		return CallNextHookEx(gs_hHook, nCode, wParam, lParam);
	}
	//

	if (IsKeyLock()) {
		// L[bNԂ̎

		// ݒ̓ǂݍ
		if (! s_KeyLockPref.IsInitialized()) {
			// 񎞁CCœǂݍ񂾐ݒRs[

			gs_KeyLockPref.Lock();
				s_KeyLockPref = gs_KeyLockPref;
				gs_KeyLockPref.SetModified(FALSE);
			gs_KeyLockPref.Unlock();

			s_dwState = STATE_IN_INIT;
		}
		else {
			gs_KeyLockPref.Lock();
			if (gs_KeyLockPref.IsModified()) {
				// Cœǂݍ񂾐ݒ肪ύXĂCēxRs[
				s_KeyLockPref = gs_KeyLockPref;
				gs_KeyLockPref.SetModified(FALSE);

				s_dwState = STATE_IN_INIT;
			}
			gs_KeyLockPref.Unlock();
		}
		//

		if (wParam == WM_KEYUP || wParam == WM_SYSKEYUP) {

			// L[bÑAvN
			for (i = 0; i < sizeof(execTable)/sizeof(execTable[0]); i++) {
				if (execTable[i].vkCode == PKBDLLHOOKSTRUCT(lParam)->vkCode) {
					if (s_KeyLockPref.m_dwExecFlag & execTable[i].dwExecFlag) {

						if (PKBDLLHOOKSTRUCT(lParam)->vkCode != s_KeyLockPref.m_dwUnlockKey1) {
							if (s_KeyLockPref.m_dwNumLockSeq > 1) {
								if (PKBDLLHOOKSTRUCT(lParam)->vkCode != s_KeyLockPref.m_dwUnlockKey2) {
									NotifyExec(	execTable[i].dwExecFlag );
								}
							}
							else {
								NotifyExec(	execTable[i].dwExecFlag );
							}
						}
					}

					break;
				}
			}
			//

			// Xe[^X^CAEg`FbN
			if (s_dwState == STATE_IN_FIRSTBTN_PRESSED &&
				(GetTickCount() - s_dwStartTick) > WZERO3_KEYLOCK_THRESHOLD) {
				s_dwState = STATE_IN_INIT;
			}
			//

			// bÑ`FbN(ԑJ)
			switch(s_dwState) {
			case STATE_IN_INIT:
				if (PKBDLLHOOKSTRUCT(lParam)->vkCode == s_KeyLockPref.m_dwUnlockKey1) {
					if (s_KeyLockPref.m_dwNumLockSeq > 1) {
						// L[bN̉2̃{^Kvꍇ
						s_dwStartTick = GetTickCount();
						s_dwState = STATE_IN_FIRSTBTN_PRESSED;
					}
					else {
						if (s_KeyLockPref.m_dwUnlockKey1 == VK_APP1 ||
							s_KeyLockPref.m_dwUnlockKey1 == VK_APP2 ||
							s_KeyLockPref.m_dwUnlockKey1 == VK_APP3
						)
						{
							// VK_APPxɂĂLWIN͎̂Ă
							s_dwState = STATE_IN_IGNORE_REST;
						}
						else {
							NotifyUnlock();
							s_dwState = STATE_IN_INIT;
						}
					}
				}
				break;

			case STATE_IN_FIRSTBTN_PRESSED:
				if (PKBDLLHOOKSTRUCT(lParam)->vkCode == s_KeyLockPref.m_dwUnlockKey2) {
					if ((GetTickCount() - s_dwStartTick) <= WZERO3_KEYLOCK_THRESHOLD) {
						if (s_KeyLockPref.m_dwUnlockKey2 == VK_APP1 ||
							s_KeyLockPref.m_dwUnlockKey2 == VK_APP2 ||
							s_KeyLockPref.m_dwUnlockKey2 == VK_APP3
						)
						{
							// VK_APPxɂĂLWIN͎̂Ă
							s_dwState = STATE_IN_IGNORE_REST;
						}
						else {
							NotifyUnlock();
							s_dwState = STATE_IN_INIT;
						}
					}
				}
				break;

			case STATE_IN_IGNORE_REST:
				NotifyUnlock();
				s_dwState = STATE_IN_INIT;
				break;
			}

			//
			if (s_KeyLockPref.IsKeepSuspend()) {
				// L[bÑTXyh̓L[vꍇ

				if (PKBDLLHOOKSTRUCT(lParam)->vkCode == s_KeyLockPref.m_dwUnlockKey1) {
					NotifyPreventSuspend();
				}
				else if (	s_KeyLockPref.m_dwNumLockSeq > 1 &&
							PKBDLLHOOKSTRUCT(lParam)->vkCode == s_KeyLockPref.m_dwUnlockKey2 ) {
					NotifyPreventSuspend();
				}
				else {
					for (i = 0; i < sizeof(execTable)/sizeof(execTable[0]); i++) {
						if (execTable[i].vkCode == PKBDLLHOOKSTRUCT(lParam)->vkCode) {
							if (s_KeyLockPref.m_dwExecFlag & execTable[i].dwExecFlag) {
								NotifyPreventSuspend();
								break;
							}
						}
					}
				}
			}
			//

		}
		//

		return 1;
	}

	return CallNextHookEx(gs_hHook, nCode, wParam, lParam);
}

static
void
HookKeyHookProc(
	DWORD	dwAction,
	void*	pData
)
{
	switch(dwAction) {
	case HKH_HOOK_ONHOOK:
		if (IsDenwaActive()) {
			CHookKeyHook::OnHook();
		}
		else {
			if (!IsKeyLock()) {
				// L[bNł͂Ȃ
				if (!ExecUserAction(HOOKKEYHOOK_SETTING_REGKEY, HOOKKEYHOOK_ONHOOK_PATH, HOOKKEYHOOK_ONHOOK_ARG)) {
					// [U`̃ANVȂ΃IWiOnHook
					CHookKeyHook::OnHook();
				}
			}
		}
		break;

	case HKH_HOOK_OFFHOOK:
		if (IsDenwaActive()) {
			CHookKeyHook::OffHook();
		}
		else {
			if (!IsKeyLock()) {
				// L[bNł͂Ȃ
				if (!ExecUserAction(HOOKKEYHOOK_SETTING_REGKEY, HOOKKEYHOOK_OFFHOOK_PATH, HOOKKEYHOOK_OFFHOOK_ARG)) {
					// [U`̃ANVȂ΃IWiOffHook
					CHookKeyHook::OffHook();
				}
			}
		}
		break;

	case HKH_HOOK_SLIDEOPEN:
		// L[{[hJ
		ExecUserAction(HOOKKEYHOOK_SETTING_REGKEY, HOOKKEYHOOK_SLIDEOPEN_PATH, HOOKKEYHOOK_SLIDEOPEN_ARG);
		break;

	case HKH_HOOK_SLIDECLOSE:
		// L[{[h
		ExecUserAction(HOOKKEYHOOK_SETTING_REGKEY, HOOKKEYHOOK_SLIDECLOSE_PATH, HOOKKEYHOOK_SLIDECLOSE_ARG);
		break;
	}
}

static
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS	wc;

    wc.style			= 0;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= 0;
    wc.hCursor			= 0;
    wc.hbrBackground	= 0;
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

static
BOOL
InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND	hWnd = NULL;
	TCHAR	szTitle[64];
	TCHAR	szWindowClass[64];

	gs_hInst = hInstance;

	lstrcpy(szWindowClass,	WZERO3_KEYLOCK_WINDOW_CLASS_NAME);
	lstrcpy(szTitle,		WZERO3_KEYLOCK_WINDOW_TITLE_NAME);

	MyRegisterClass(hInstance, szWindowClass);
	
	hWnd = CreateWindow(szWindowClass, szTitle, 0, 0, 0, 0, 0, NULL, NULL, hInstance, NULL);
	if (!hWnd) {	
		return FALSE;
	}

	return TRUE;
}


int
WINAPI
WinMain(
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPTSTR    lpCmdLine,
	int       nCmdShow
)
{
	MSG				msg;
	HANDLE			hMutex = NULL;

	// dN`FbN
	hMutex = CreateMutex(NULL, TRUE, WZERO3_KEYLOCK_APP_MUTEX_NAME);
	if (hMutex != NULL) {
		if (GetLastError() == ERROR_ALREADY_EXISTS) {
			int		iRet;

			iRet = Util_MessageBox(NULL, hInstance, IDS_03KEYLOCK_CONFIRM, IDS_03KEYLOCK_CAPTION, MB_YESNO|MB_ICONINFORMATION|MB_SETFOREGROUND);

			if (iRet == IDYES) {
				HWND	hWnd;

				hWnd = FindWindow(WZERO3_KEYLOCK_WINDOW_CLASS_NAME, WZERO3_KEYLOCK_WINDOW_TITLE_NAME);
				if (hWnd != NULL) {
					PostMessage(hWnd, WM_CLOSE, 0, 0);
				}
			}

			CloseHandle(hMutex);
			hMutex = NULL;
			return FALSE;
		}
	}
	else {
		return FALSE;
	}
	//
	// Perform application initialization:
	if (!InitInstance(hInstance, nCmdShow)) {
		ReleaseMutex(hMutex);
		CloseHandle(hMutex);
		hMutex = NULL;
		return FALSE;
	}

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	ReleaseMutex(hMutex);
	CloseHandle(hMutex);
	hMutex = NULL;
	return msg.wParam;
}

static
LRESULT
CALLBACK
WndProc(
	HWND	hWnd,
	UINT	message,
	WPARAM	wParam,
	LPARAM	lParam
)
{

	static	BOOL	s_bKeyHookSucc			= FALSE;
	static	BOOL	s_bHookKeyHookSucc		= FALSE;
	static	BOOL	s_bPowerWatchSucc		= FALSE;
	static	BOOL	s_bCanResuspend			= FALSE;
	static	CHookKeyHook	s_hookKeyHook;
	static	CPowerWatch		s_PowerWatch;

	switch (message) 
	{
		case WM_CREATE:
			{
				gs_KeyLockPref.Lock();
					gs_KeyLockPref.Load();
				gs_KeyLockPref.Unlock();

				// L[{[h̃tbN
				s_bKeyHookSucc = StartHook(HookProc);
				if (s_bKeyHookSucc) {
					// OnHook/OffHookL[̃tbN
					s_bHookKeyHookSucc = s_hookKeyHook.StartHook(HKH_HOOK_ONHOOK|HKH_HOOK_OFFHOOK|HKH_HOOK_SLIDEOPEN|HKH_HOOK_SLIDECLOSE, HookKeyHookProc, NULL);
					if (s_bHookKeyHookSucc) {
						if (gs_KeyLockPref.IsKeepSuspend()) {
							// TXyhL[v@\L
							s_bPowerWatchSucc = s_PowerWatch.StartWatch(NotifyResumeProc, hWnd);
							if (s_bPowerWatchSucc) {
									;
							}
							else {
								s_hookKeyHook.StopHook();
								s_bHookKeyHookSucc = FALSE;

								StopHook();
								s_bKeyHookSucc = FALSE;

								MessageBox(NULL, _T("PowerWatch.StartWatch fail"), WZERO3_KEYLOCK_APP_DBG_CAPTION, MB_OK);
								return -1;
							}
						}
					}
					else {

						StopHook();
						s_bKeyHookSucc = FALSE;

						MessageBox(NULL, _T("HookKeyHook.StartHook fail"), WZERO3_KEYLOCK_APP_DBG_CAPTION, MB_OK);
						return -1;
					}
				}
				else {
					MessageBox(NULL, _T("StartHook fail"), WZERO3_KEYLOCK_APP_DBG_CAPTION, MB_OK);
					return -1;
				}
			}
			break;

		case WM_CLOSE:
			DestroyWindow(hWnd);
			break;

		case WM_DESTROY:
			{
				KillTimer(hWnd, TIMERID_WAIT_CHECKKEYLOCK);
				KillTimer(hWnd, TIMERID_WAIT_RESUSPEND);

				// Ô߃L[bN͉Ă
				doKeyUnlock();
				//

				if (s_bPowerWatchSucc) {
					s_PowerWatch.StopWatch();
					s_bPowerWatchSucc = FALSE;
				}
				if (s_bHookKeyHookSucc) {
					s_hookKeyHook.StopHook();
					s_bHookKeyHookSucc = FALSE;
				}
				if (s_bKeyHookSucc) {
					StopHook();
					s_bKeyHookSucc = FALSE;
				}

				PostQuitMessage(0);
			}
			break;

		case WM_TIMER:
			switch(wParam) {
			case TIMERID_WAIT_CHECKKEYLOCK:
				// TXyhACL[bNԃ`FbN\

				KillTimer(hWnd, TIMERID_WAIT_CHECKKEYLOCK);

				if (gs_KeyLockPref.IsKeepSuspend()) {

					if (IsKeyLock()) {
						// L[bNԂȂ΁C莞ԌɃTXyh݂
						s_bCanResuspend = TRUE;
						SetTimer(hWnd, TIMERID_WAIT_RESUSPEND, WAIT_TIME_RESUSPEND, NULL);
					}
				}

				break;

			case TIMERID_WAIT_RESUSPEND:
				// L[bÑTXyhL[v

				KillTimer(hWnd, TIMERID_WAIT_RESUSPEND);
				s_bCanResuspend = FALSE;

				if (gs_KeyLockPref.IsKeepSuspend()) {
					if (IsKeyLock()) {
						if (IsDenwaActive()		||
							IsPPPConnected()	||
							IsOutLookReceived()	) {

							// dbCPPPRlNVC[M
							// TXyhłȂCTXyhL[v͂߂...
							;
						}
						else {
							// TXyh̎s
							doSuspend();
						}
					}
				}

				break;

			default:
				return DefWindowProc(hWnd, message, wParam, lParam);
				break;
			}

			break;


		case WM_USER_KEYLOCK_CONTROL:
			{
				// L[bNԂ̃Rg[
				BOOL	bDoLock = FALSE;

				if (wParam == KEYLOCK_CONTROL_ON) {
					// L[bNON
					bDoLock = TRUE;
				}
				else if (wParam == KEYLOCK_CONTROL_OFF) {
					// L[bNOFF
					bDoLock = FALSE;
				}
				else {
					// ftHg̓gO
					bDoLock = IsKeyLock() ? FALSE : TRUE;
				}

				if (bDoLock) {
					if (!(lParam & KEYLOCK_CONTROL_OPT_NOEXEC)) {
						ExecUserAction(WZERO3_KEYLOCK_SETTING_REGKEY, WZERO3_KEYLOCK_EXEC_ON_LOCK_CMDPATH, WZERO3_KEYLOCK_EXEC_ON_LOCK_ARG);
					}
					doKeyLock();
				}
				else {
					// TXyhL[v֌W̏Ԃ̓Zbg
					KillTimer(hWnd, TIMERID_WAIT_CHECKKEYLOCK);
					KillTimer(hWnd, TIMERID_WAIT_RESUSPEND);
					s_bCanResuspend = FALSE;
					//

					doKeyUnlock();
					if (!(lParam & KEYLOCK_CONTROL_OPT_NOEXEC)) {
						ExecUserAction(WZERO3_KEYLOCK_SETTING_REGKEY, WZERO3_KEYLOCK_EXEC_ON_UNLOCK_CMDPATH, WZERO3_KEYLOCK_EXEC_ON_UNLOCK_ARG);
					}
				}
			}
			break;

		case WM_USER_RELOAD_SETTING:
			{
				// ݒ̓ǂݒv

				CKeyLockPref	keyLockPref;
				BOOL			bIsKeepSuspendOld;

				// Vݒ[h
				keyLockPref.Load();
				//

				bIsKeepSuspendOld = gs_KeyLockPref.IsKeepSuspend();

				// TXyhL[v -> TXyhL[vȂ
				// ɐݒ肪ύXɂȂꍇCTXyhĎXbh~
				if (bIsKeepSuspendOld) {
					if (!keyLockPref.IsKeepSuspend()) {
						s_PowerWatch.StopWatch();
						s_bPowerWatchSucc = FALSE;
					}
				}
				//

				// Vݒ𔽉f
				gs_KeyLockPref.Lock();
					gs_KeyLockPref = keyLockPref;
					gs_KeyLockPref.SetModified(TRUE);
				gs_KeyLockPref.Unlock();
				//

				// TXyhL[vȂ -> TXyhL[v
				// ɐݒ肪ύXɂȂꍇCTXyhĎXbhN
				if (!bIsKeepSuspendOld) {
					if (keyLockPref.IsKeepSuspend()) {
						s_bPowerWatchSucc = s_PowerWatch.StartWatch(NotifyResumeProc, hWnd);
					}
				}
				//
			}
			break;

		case WM_USER_EXEC_IN_KEYLOCK:
			{
				// L[bN̊OAvs
				int		i;
				const static struct {
					DWORD	dwExecFlag;
					LPTSTR	pCmdRegName;
					LPTSTR	pArgRegName;
				} execTable[] = {
					{	WZERO3_KEYLOCK_EXEC_VK_F1,		WZERO3_KEYLOCK_EXEC_VK_F1_CMDPATH,	WZERO3_KEYLOCK_EXEC_VK_F1_ARG	},
					{	WZERO3_KEYLOCK_EXEC_VK_F2,		WZERO3_KEYLOCK_EXEC_VK_F2_CMDPATH,	WZERO3_KEYLOCK_EXEC_VK_F2_ARG	},
					{	WZERO3_KEYLOCK_EXEC_VK_APP1,	WZERO3_KEYLOCK_EXEC_VK_APP1_CMDPATH,	WZERO3_KEYLOCK_EXEC_VK_APP1_ARG	},
					{	WZERO3_KEYLOCK_EXEC_VK_APP2,	WZERO3_KEYLOCK_EXEC_VK_APP2_CMDPATH,	WZERO3_KEYLOCK_EXEC_VK_APP2_ARG	},
					{	WZERO3_KEYLOCK_EXEC_VK_APP3,	WZERO3_KEYLOCK_EXEC_VK_APP3_CMDPATH,	WZERO3_KEYLOCK_EXEC_VK_APP3_ARG	},
				};

				for (i = 0; i < sizeof(execTable)/sizeof(execTable[0]); i++) {
					if (execTable[i].dwExecFlag == wParam) {
						ExecUserAction(WZERO3_KEYLOCK_SETTING_REGKEY, execTable[i].pCmdRegName, execTable[i].pArgRegName);
						break;
					}
				}
			}
			break;

		case WM_USER_NOTIFY_RESUME:
			// TXyh̕A

			// TXyhÁCL[bNtO
			// ݒ肳ĂȂ\邽߁CwaitD
			s_bCanResuspend = FALSE;
			SetTimer(hWnd, TIMERID_WAIT_CHECKKEYLOCK, WAIT_TIME_CHECKKEYLOCK, NULL);
			
			break;

		case WM_USER_PREVENT_SUSPEND:
			// L[ɂCTXyhL[vjQ

			if (s_bCanResuspend) {
				SetTimer(hWnd, TIMERID_WAIT_RESUSPEND, WAIT_TIME_RESUSPEND, NULL);
			}

			break;

		case WM_USER_TICK:
			// B@\
			if (IsKeyLock()) {
				// KeyLockSuspenderLockꂽƊႢ
				UnsetLockIcon();
				Sleep(200);
				SetLockIcon();
			}
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
			break;
	}

	return 0;
}
