// KeyLockPref.cpp: CKeyLockPref NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include <winuserm.h>
#include "KeyLockPref.h"
#include "03KeyLockComm.h"

///////////////////////////////////////////////////

static
BOOL
IsValidLockKey(
	DWORD dwKey
)
{
	if (dwKey == VK_F1 ||	// \tgL[1(0x70)
		dwKey == VK_F2 ||	// \tgL[2(0x71)
		dwKey == VK_APP1 || // IE{^(0xC1)
		dwKey == VK_APP2 || // [{^(0xC2)
		dwKey == VK_APP3    // J{^(0xC3)
	) {
		return TRUE;
	}

	return FALSE;
}
//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CKeyLockPref::CKeyLockPref()
{
	InitializeCriticalSection(&m_Section);

	m_bInitialized = FALSE;
	m_bModified	= FALSE;

	m_dwNumLockSeq = 2;
	m_dwUnlockKey1 = VK_F1;		// \tgL[1
	m_dwUnlockKey2 = VK_APP1;	// IEL[

	m_dwExecFlag = 0;

	m_bKeepSuspend = FALSE;
}

CKeyLockPref::CKeyLockPref(
	const CKeyLockPref& keyLockPref
)
{
	InitializeCriticalSection(&m_Section);

	m_bInitialized	= keyLockPref.m_bInitialized;
	m_bModified		= FALSE;

	m_dwNumLockSeq = keyLockPref.m_dwNumLockSeq;
	m_dwUnlockKey1 = keyLockPref.m_dwUnlockKey1;
	m_dwUnlockKey2 = keyLockPref.m_dwUnlockKey2;

	m_dwExecFlag = keyLockPref.m_dwExecFlag;

	m_bKeepSuspend = keyLockPref.m_bKeepSuspend;
}

CKeyLockPref::~CKeyLockPref()
{
	DeleteCriticalSection(&m_Section);
}

CKeyLockPref& 
CKeyLockPref::operator=(
	const CKeyLockPref& keyLockPref
)
{
	m_bInitialized	= keyLockPref.m_bInitialized;
	m_bModified		= FALSE;

	m_dwNumLockSeq = keyLockPref.m_dwNumLockSeq;
	m_dwUnlockKey1 = keyLockPref.m_dwUnlockKey1;
	m_dwUnlockKey2 = keyLockPref.m_dwUnlockKey2;

	m_dwExecFlag = keyLockPref.m_dwExecFlag;

	m_bKeepSuspend = keyLockPref.m_bKeepSuspend;

	return *this;
}

void
CKeyLockPref::Lock()
{
	EnterCriticalSection(&m_Section);
}
void

CKeyLockPref::Unlock()
{
	LeaveCriticalSection(&m_Section);
}

DWORD
static
GetExecFlag(
	HKEY	hKey
)
{
	int	i;
	DWORD	dwRetFlag;
	const struct {
		LPTSTR	pCmdRegName;
		DWORD	dwFlag;
	} checkTable[] = {
		{	WZERO3_KEYLOCK_EXEC_VK_F1_CMDPATH,		WZERO3_KEYLOCK_EXEC_VK_F1	},
		{	WZERO3_KEYLOCK_EXEC_VK_F2_CMDPATH,		WZERO3_KEYLOCK_EXEC_VK_F2	},
		{	WZERO3_KEYLOCK_EXEC_VK_APP1_CMDPATH,	WZERO3_KEYLOCK_EXEC_VK_APP1	},
		{	WZERO3_KEYLOCK_EXEC_VK_APP2_CMDPATH,	WZERO3_KEYLOCK_EXEC_VK_APP2	},
		{	WZERO3_KEYLOCK_EXEC_VK_APP3_CMDPATH,	WZERO3_KEYLOCK_EXEC_VK_APP3	},
	};

	dwRetFlag = 0;	// return value

	for (i = 0; i < sizeof(checkTable)/sizeof(checkTable[0]); i++) {
		TCHAR	szPath[MAX_PATH + 1];
		DWORD	dwType;
		DWORD	dwSize;
		LONG	lResult;

		dwType = 0;
		dwSize = sizeof(szPath);
		memset(szPath, 0, sizeof(szPath));
		lResult = RegQueryValueEx(hKey, checkTable[i].pCmdRegName, NULL, &dwType, (LPBYTE)szPath, &dwSize);
		if (lResult == ERROR_SUCCESS) {
			if (_tcslen(szPath) != 0) {
				dwRetFlag |= checkTable[i].dwFlag;
			}
		}
	}

	return dwRetFlag;
}

void
CKeyLockPref::Load()
{
	LONG	lResult;
	HKEY	hKey			= NULL;
	DWORD	dwUnlockKey1	= 0;
	DWORD	dwUnlockKey2	= 0;
	DWORD	dwSuccNum		= 0;
	DWORD	dwExecFlag		= 0;
	DWORD	dwKeepSuspend	= FALSE;

	dwSuccNum = 0;
	lResult = RegOpenKeyEx(HKEY_CURRENT_USER, WZERO3_KEYLOCK_SETTING_REGKEY, 0, KEY_ALL_ACCESS, &hKey);

	if (lResult == ERROR_SUCCESS) {

		DWORD	dwSize;
		DWORD	dwType;

		// bNL[̐ݒ
		dwType = 0;
		dwSize = sizeof(dwUnlockKey1);

		lResult = RegQueryValueEx(hKey, WZERO3_KEYLOCK_UNLOCKKEY1_REGVAL, NULL, &dwType, (LPBYTE)&dwUnlockKey1, &dwSize);
		if (lResult == ERROR_SUCCESS) {
			if (IsValidLockKey(dwUnlockKey1)) {

				dwSuccNum++;

				dwType = 0;
				dwSize = sizeof(dwUnlockKey2);

				lResult = RegQueryValueEx(hKey, WZERO3_KEYLOCK_UNLOCKKEY2_REGVAL, NULL, &dwType, (LPBYTE)&dwUnlockKey2, &dwSize);
				if (lResult == ERROR_SUCCESS) {
					if (IsValidLockKey(dwUnlockKey2)) {
						dwSuccNum++;
					}
					else {
						// unlockkey2̒lꍇCݒftHgɖ߂
						dwSuccNum = 0;
					}
				}

			}
		}
		//

		// L[bÑL[ɂOAvN
		dwExecFlag = GetExecFlag(hKey);
		//

		// L[bÑTXyhL[v邩
		dwType = 0;
		dwSize = sizeof(dwKeepSuspend);
		lResult = RegQueryValueEx(hKey, WZERO3_KEYLOCK_KEEPSUSPEND, NULL, &dwType, (LPBYTE)&dwKeepSuspend, &dwSize);
		if (lResult != ERROR_SUCCESS) {
			dwKeepSuspend = 0;
		}
		//

		RegCloseKey(hKey);
		hKey = NULL;
	}

	switch(dwSuccNum) {
	case 0:
		{
			// ݒ肪WXgɏ܂ĂȂC͐ݒls̏ꍇC
			// ftHg̐ݒƂD
			m_dwNumLockSeq = 2;
			m_dwUnlockKey1 = VK_F1;		// \tgL[1
			m_dwUnlockKey2 = VK_APP1;	// IEL[
		}
		break;
	case 1:
		{
			m_dwNumLockSeq = 1;
			m_dwUnlockKey1 = dwUnlockKey1;
		}
		break;
	case 2:
		{
			m_dwNumLockSeq = 2;
			m_dwUnlockKey1 = dwUnlockKey1;
			m_dwUnlockKey2 = dwUnlockKey2;
		}
		break;
	}

	m_dwExecFlag = dwExecFlag;
	m_bKeepSuspend = ((dwKeepSuspend != 0) ? TRUE : FALSE);

	m_bInitialized = TRUE;
	m_bModified = TRUE;

}

BOOL
CKeyLockPref::IsInitialized()
{
	return m_bInitialized;
}

BOOL
CKeyLockPref::IsModified()
{
	return m_bModified;
}

void
CKeyLockPref::SetModified(
	BOOL bModified
)
{
	m_bModified = bModified;
}

BOOL
CKeyLockPref::IsKeepSuspend()
{
	return m_bKeepSuspend;
}